package com.myeclipseide.example.rest;

import java.util.List;

import javax.persistence.EntityManager;

import com.myeclipseide.example.jpa.EntityManagerHelper;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;

/**
 * @author MyEclipse Web Service Tools
 */
public abstract class AbstractFacade<T> {
	private Class<T> entityClass;

	public AbstractFacade(Class<T> entityClass) {
		this.entityClass = entityClass;
	}

	protected abstract EntityManager getEntityManager();

	public Response create(T entity) {
		try {
			if (!getEntityManager().getTransaction().isActive()) {
				getEntityManager().getTransaction().begin();
			}
			getEntityManager().persist(entity);
			EntityManagerHelper.commit();
			return Response.created(null).entity("Entity created successfully")
					.build();
		} catch (Exception ex) {
			throw new WebApplicationException(ex, Response.Status.BAD_REQUEST);
		}
	}

	public Response edit(T entity) {
		try {
			if (!getEntityManager().getTransaction().isActive()) {
				getEntityManager().getTransaction().begin();
			}
			getEntityManager().merge(entity);
			EntityManagerHelper.commit();
			return Response.ok().entity("Entity edited successfully").build();
		} catch (Exception ex) {
			throw new WebApplicationException(ex, Response.Status.NOT_FOUND);
		}
	}

	public Response remove(T entity) {
		try {
			if (!getEntityManager().getTransaction().isActive()) {
				getEntityManager().getTransaction().begin();
			}
			getEntityManager().remove(getEntityManager().merge(entity));
			EntityManagerHelper.commit();
			return Response.ok().entity("Entity deleted successfully").build();
		} catch (Exception ex) {
			throw new WebApplicationException(ex, Response.Status.NOT_FOUND);
		}
	}

	public T find(Object id) {
		return getEntityManager().find(entityClass, id);
	}

	public List<T> findAll() {
		javax.persistence.criteria.CriteriaQuery cq = getEntityManager()
				.getCriteriaBuilder().createQuery();
		cq.select(cq.from(entityClass));
		return getEntityManager().createQuery(cq).getResultList();
	}

	public List<T> findRange(int[] range) {
		javax.persistence.criteria.CriteriaQuery cq = getEntityManager()
				.getCriteriaBuilder().createQuery();
		cq.select(cq.from(entityClass));
		javax.persistence.Query q = getEntityManager().createQuery(cq);
		q.setMaxResults(range[1] - range[0]);
		q.setFirstResult(range[0]);
		return q.getResultList();
	}

	public int count() {
		javax.persistence.criteria.CriteriaQuery cq = getEntityManager()
				.getCriteriaBuilder().createQuery();
		javax.persistence.criteria.Root<T> rt = cq.from(entityClass);
		cq.select(getEntityManager().getCriteriaBuilder().count(rt));
		javax.persistence.Query q = getEntityManager().createQuery(cq);
		return ((Long) q.getSingleResult()).intValue();
	}

}
